/***************************************************************************/
/* The purpose of this macro is to create a report of the portfolio */
/* optimization process, subject to a given risk level. */
/* Parameters: */
/* inDir : Directory of SAS Risk Dimensions */
/* SAS library */
/* env_name: SAS Risk Dimensions environment */
/* Proj : SAS Risk Dimensions project for the report */
/* es : Expected shortfall */
/***************************************************************************/
%macro WriteReport(inDir, env_name, Proj, es);
/* Libref to project output. */
%let inpath=%sysfunc(pathname(&inDir,L));
%if %index(&inpath,\) %then %let sep=\; %else %let sep=/;
libname projdir "&inpath.&sep.&env_name.&sep.&Proj";
proc sql;
/* Create a new data set containing the optimal weights. */
  create table &inDir..OptWgt as
    select Company,
  OptWeight as wgt "Optimal Weight"
  from projdir.optvalue
  where Company <> "+";
quit;
/*Use the Portfolio Generation macro to create the optimal portfolio.*/
%CreateInstSet(&inDir,Mkt_Data,OptWgt,Inst_Data_wo,Opt_Inst_Data,value=&pValue);
proc sql;
/* Create a new table to contain the relevant information */
/* for the report. */
  create table &indir..OptReport
   (Company Char(15) label="Company*Name",
   Holding num label="Optimal*Holding",
   opt_wgt num label="Optimal*Weight",
   old_wgt num label="Original*Weight",
   old_holding num label="Original*Holding",
   amt_sell num label="Amount*to Sell"
   );
/* Populate the table with the information. */
insert into &inDir..OptReport (Company,
  Holding,
  opt_wgt, old_wgt,
  old_holding, amt_sell)
select a.Company,
       a.Holding,
       b.wgt,
       c.wgt,
       d.Holding,
       (d.Holding - a.Holding)
  from &inDir..Opt_Inst_Data as a,
       &inDir..OptWgt as b,
       &inDir..wght as c,
       &inDir..Inst_Data as d
where (a.Company = b.Company) AND
      (b.Company = c.Company) AND
      (c.Company = d.Company);
quit;
title "Project: &Proj";
title2 "Constraint: Expected Shortfall = &ES";
/* Use PROC REPORT to generate the formatted report. */
proc report data = &inDir..OptReport split="*";
   column Company old_wgt opt_wgt old_holding Holding
   to_sell to_buy;
   define old_wgt / display format=6.4;
   define opt_wgt / display format=6.4;
   define old_holding / display;
   define Holding / display;
   define to_sell / computed "Amount*to Sell"
   format=6.0;
   define to_buy / computed "Amount*to Buy"
   format=6.0;
   compute to_sell;
   to_sell = old_holding - Holding;
   if to_sell < 0 then to_sell = 0;
   endcomp;
   compute to_buy;
   to_buy = Holding - old_holding;
   if to_buy < 0 then to_buy = 0;
   endcomp;
run;
title;
proc report data = projdir.optsum split="*";
column AnalysisName BaseDate _date_ objectivename constraintname constraintRHS CurObjective OptObjective;
define analysisname / display "Name*of*Analysis";
define basedate / display "Base*Case*Date";
define _date_ / display "Valuation*Date";
define objectivename / display "Objective";
define constraintname / display "Constraint";
define constraintRHS / display "Constraint*=";
define CurObjective / display "Original*Expected*Return" format=6.4;
define OptObjective / display "Optimal*Expected*Return" format=6.4;
run;
libname projdir clear;
%mend WriteReport;
